@@ -2,15 +2,13 @@ use strict;
 
 package Pod::Coverage;
 use Devel::Symdump;
+use B;
 use Pod::Find qw(pod_where);
 
-BEGIN { defined &TRACE_ALL or eval 'sub TRACE_ALL () { 0 }' };
-
-use DynaLoader ();
-use base 'DynaLoader';
+BEGIN { defined &TRACE_ALL or eval 'sub TRACE_ALL () { 0 }' }
 
 use vars qw/ $VERSION /;
-$VERSION = '0.17';
+$VERSION = '0.19';
 
 =head1 NAME
 
@@ -73,10 +71,22 @@ Creates a new Pod::Coverage object.
 C<package> the name of the package to analyse
 
 C<private> an array of regexen which define what symbols are regarded
-as private (and so need not be documented) defaults to /^_/,
-/^import$/, /^DESTROY$/, /^AUTOLOAD$/, /^bootstrap$/, /^(TIE(SCALAR|ARRAY|HASH|HANDLE)|FETCH|STORE|UNTIE|FETCHSIZE|STORESIZE|POP|PUSH|SHIFT|UNSHIFT|SPLICE|DELETE|EXISTS|EXTEND|CLEAR|FIRSTKEY|NEXTKEY|PRINT|PRINTF|WRITE|READLINE|GETC|READ|CLOSE|BINMODE|OPEN|EOF|FILENO|SEEK|TELL)$/.  That last big one covers all the
-required and optional methods for tie()d objects, as these methods are
-(hardly) ever called by a user, being used internally by perl.
+as private (and so need not be documented) defaults to [ qr/^_/,
+qr/^import$/, qr/^DESTROY$/, qr/^AUTOLOAD$/, qr/^bootstrap$/,
+        qr/^(TIE( SCALAR | ARRAY | HASH | HANDLE ) |
+             FETCH | STORE | UNTIE | FETCHSIZE | STORESIZE |
+             POP | PUSH | SHIFT | UNSHIFT | SPLICE | DELETE |
+             EXISTS | EXTEND | CLEAR | FIRSTKEY | NEXTKEY | PRINT | PRINTF |
+             WRITE | READLINE | GETC | READ | CLOSE | BINMODE | OPEN |
+             EOF | FILENO | SEEK | TELL)$/x,
+        qr/^( MODIFY | FETCH )_( REF | SCALAR | ARRAY | HASH | CODE |
+                                 GLOB | FORMAT | IO)_ATTRIBUTES$/x,
+        qr/^CLONE(_SKIP)?$/,
+]
+
+This should cover all the usual magical methods for tie()d objects,
+attributes, generally all the methods that are typically not called by
+a user, but instead being used internally by perl.
 
 C<also_private> items are appended to the private list
 
@@ -87,6 +97,9 @@ for them
 If C<pod_from> is supplied, that file is parsed for the documentation,
 rather than using Pod::Find
 
+If C<nonwhitespace> is supplied, then only POD sections which have
+non-whitespace characters will count towards being documented.
+
 =cut
 
 sub new {
@@ -101,12 +114,26 @@ sub new {
         qr/^AUTOLOAD$/,
         qr/^bootstrap$/,
         qr/^\(/,
-        qr/^(TIE(SCALAR|ARRAY|HASH|HANDLE)|FETCH|STORE|UNTIE|FETCHSIZE|STORESIZE|POP|PUSH|SHIFT|UNSHIFT|SPLICE|DELETE|EXISTS|EXTEND|CLEAR|FIRSTKEY|NEXTKEY|PRINT|PRINTF|WRITE|READLINE|GETC|READ|CLOSE|BINMODE|OPEN|EOF|FILENO|SEEK|TELL)$/
-       ];
+        qr/^(TIE( SCALAR | ARRAY | HASH | HANDLE ) |
+             FETCH | STORE | UNTIE | FETCHSIZE | STORESIZE |
+             POP | PUSH | SHIFT | UNSHIFT | SPLICE | DELETE |
+             EXISTS | EXTEND | CLEAR | FIRSTKEY | NEXTKEY | PRINT | PRINTF |
+             WRITE | READLINE | GETC | READ | CLOSE | BINMODE | OPEN |
+             EOF | FILENO | SEEK | TELL)$/x,
+        qr/^( MODIFY | FETCH )_( REF | SCALAR | ARRAY | HASH | CODE |
+                                 GLOB | FORMAT | IO)_ATTRIBUTES $/x,
+        qr/^CLONE(_SKIP)?$/,
+    ];
     push @$private, @{ $args{also_private} || [] };
-    my $trustme = $args{trustme} || [];
+    my $trustme       = $args{trustme}       || [];
+    my $nonwhitespace = $args{nonwhitespace} || undef;
 
-    my $self = bless { @_, private => $private, trustme => $trustme }, $class;
+    my $self = bless {
+        @_,
+        private       => $private,
+        trustme       => $trustme,
+        nonwhitespace => $nonwhitespace
+    }, $class;
 }
 
 =item $object->coverage
@@ -115,22 +142,21 @@ Gives the coverage as a value in the range 0 to 1
 
 =cut
 
-
 sub coverage {
     my $self = shift;
 
     my $package = $self->{package};
-    my $pods = $self->_get_pods;
+    my $pods    = $self->_get_pods;
     return unless $pods;
 
     my %symbols = map { $_ => 0 } $self->_get_syms($package);
 
     print "tying shoelaces\n" if TRACE_ALL;
-    for my $pod ( @$pods ) {
+    for my $pod (@$pods) {
         $symbols{$pod} = 1 if exists $symbols{$pod};
     }
 
-    foreach my $sym (keys %symbols) {
+    foreach my $sym ( keys %symbols ) {
         $symbols{$sym} = 1 if $self->_trustme_check($sym);
     }
 
@@ -142,8 +168,8 @@ sub coverage {
         print Data::Dumper::Dumper($self);
     }
 
-    my $symbols    = scalar keys %symbols;
-    my $documented = scalar grep { $_ } values %symbols;
+    my $symbols = scalar keys %symbols;
+    my $documented = scalar grep {$_} values %symbols;
     unless ($symbols) {
         $self->{why_unrated} = "no public symbols defined";
         return;
@@ -161,10 +187,9 @@ be able to check C<why_unrated> to get a useful excuse.
 
 sub why_unrated {
     my $self = shift;
-    $self->{why_unrated}
+    $self->{why_unrated};
 }
 
-
 =item $object->naked/$object->uncovered
 
 Returns a list of uncovered routines, will implicitly call coverage if
@@ -204,19 +229,19 @@ sub import {
     return unless @_;
 
     # one argument - just a package
-    scalar @_ == 1  and  unshift @_, 'package';
+    scalar @_ == 1 and unshift @_, 'package';
 
     # we were called with arguments
     my $pc     = $self->new(@_);
     my $rating = $pc->coverage;
-    $rating = 'unrated ('. $pc->why_unrated .')'
-      unless defined $rating;
+    $rating = 'unrated (' . $pc->why_unrated . ')'
+        unless defined $rating;
     print $pc->{package}, " has a $self rating of $rating\n";
     my @looky_here = $pc->naked;
     if ( @looky_here > 1 ) {
-        print "The following are uncovered: ", join(", ", sort @looky_here), "\n";
-    }
-    elsif (@looky_here) {
+        print "The following are uncovered: ", join( ", ", sort @looky_here ),
+            "\n";
+    } elsif (@looky_here) {
         print "'$looky_here[0]' is uncovered\n";
     }
 }
@@ -276,6 +301,7 @@ sub _get_syms {
 
     print "requiring '$package'\n" if TRACE_ALL;
     eval qq{ require $package };
+    print "require failed with $@\n" if TRACE_ALL and $@;
     return if $@;
 
     print "walking symbols\n" if TRACE_ALL;
@@ -283,10 +309,14 @@ sub _get_syms {
 
     my @symbols;
     for my $sym ( $syms->functions ) {
+
         # see if said method wasn't just imported from elsewhere
-        my $owner = $self->_CvGV( \&{ $sym } );
-        $owner =~ s/^\*(.*)::.*?$/$1/;
-        next if $owner ne $self->{package};
+        my $glob = do { no strict 'refs'; \*{$sym} };
+        my $o = B::svref_2object($glob);
+
+        # in 5.005 this flag is not exposed via B, though it exists
+        my $imported_cv = eval { B::GVf_IMPORTED_CV() } || 0x80;
+        next if $o->GvFLAGS & $imported_cv;
 
         # check if it's on the whitelist
         $sym =~ s/$self->{package}:://;
@@ -321,12 +351,12 @@ sub _get_pods {
 
     print "parsing '$pod_from'\n" if TRACE_ALL;
     my $pod = Pod::Coverage::Extractor->new;
+    $pod->{nonwhitespace} = $self->{nonwhitespace};
     $pod->parse_from_file( $pod_from, '/dev/null' );
 
     return $pod->{identifiers} || [];
 }
 
-
 =item _private_check($symbol)
 
 return true if the symbol should be considered private
@@ -335,7 +365,7 @@ return true if the symbol should be considered private
 
 sub _private_check {
     my $self = shift;
-    my $sym = shift;
+    my $sym  = shift;
     return grep { $sym =~ /$_/ } @{ $self->{private} };
 }
 
@@ -346,44 +376,72 @@ return true if the symbol is a 'trustme' symbol
 =cut
 
 sub _trustme_check {
-    my($self, $sym) = @_;
-    return grep { $sym =~ /$_/ } @{$self->{trustme} };
+    my ( $self, $sym ) = @_;
+    return grep { $sym =~ /$_/ } @{ $self->{trustme} };
 }
 
-bootstrap Pod::Coverage;
+sub _CvGV {
+    my $self = shift;
+    my $cv   = shift;
+    my $b_cv = B::svref_2object($cv);
 
+    # perl 5.6.2's B doesn't have an object_2svref.  in 5.8 you can
+    # just do this:
+    # return *{ $b_cv->GV->object_2svref };
+    # but for backcompat we're forced into this uglyness:
+    no strict 'refs';
+    return *{ $b_cv->GV->STASH->NAME . "::" . $b_cv->GV->NAME };
+}
 
 package Pod::Coverage::Extractor;
 use Pod::Parser;
 use base 'Pod::Parser';
 
 use constant debug => 0;
+
 # extract subnames from a pod stream
 sub command {
     my $self = shift;
-    my ($command, $text, $line_num) = @_;
-    if ($command eq 'item' || $command =~ /^head(?:2|3|4)/) {
+    my ( $command, $text, $line_num ) = @_;
+    if ( $command eq 'item' || $command =~ /^head(?:2|3|4)/ ) {
+
         # take a closer look
-        my @pods = ($text =~ /\s*([^\s\|,\/]+)/g);
+        my @pods = ( $text =~ /\s*([^\s\|,\/]+)/g );
+        $self->{recent} = [];
 
         foreach my $pod (@pods) {
             print "Considering: '$pod'\n" if debug;
 
             # it's dressed up like a method cal
-            $pod =~ /-E<\s*gt\s*>(.*)/  and $pod = $1;
-            $pod =~ /->(.*)/            and $pod = $1;
+            $pod =~ /-E<\s*gt\s*>(.*)/ and $pod = $1;
+            $pod =~ /->(.*)/           and $pod = $1;
+
+            # it's used as a (bare) fully qualified name
+            $pod =~ /\w+(?:::\w+)*::(\w+)/ and $pod = $1;
+
             # it's wrapped in a pod style B<>
             $pod =~ s/[A-Z]<//g;
             $pod =~ s/>//g;
+
             # has arguments, or a semicolon
-            $pod =~ /(\w+)\s*[;\(]/   and $pod = $1;
+            $pod =~ /(\w+)\s*[;\(]/ and $pod = $1;
 
             print "Adding: '$pod'\n" if debug;
-            push @{$self->{identifiers}}, $pod;
+            push @{ $self->{ $self->{nonwhitespace}
+                    ? "recent"
+                    : "identifiers" } }, $pod;
         }
     }
 }
 
+sub textblock {
+    my $self = shift;
+    my ( $text, $line_num ) = shift;
+    if ( $self->{nonwhitespace} and $text =~ /\S/ and $self->{recent} ) {
+        push @{ $self->{identifiers} }, @{ $self->{recent} };
+        $self->{recent} = [];
+    }
+}
 
 1;
 
@@ -421,8 +479,8 @@ some contributions from David Cantrell <david@cantrell.org.uk>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001, 2003, 2004 Richard Clamp, Michael Stevens. All
-rights reserved.  This program is free software; you can redistribute
-it and/or modify it under the same terms as Perl itself.
+Copyright (c) 2001, 2003, 2004, 2006, 2007 Richard Clamp, Michael
+Stevens. All rights reserved.  This program is free software; you can
+redistribute it and/or modify it under the same terms as Perl itself.
 
 =cut
